@@ -1,4 +1,4 @@
-#
+
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -19,8 +19,9 @@ Not implemented yet:
       - placeholders are ignored
 """
 import atexit
-import ConfigParser
+import contextlib
 import errno
+import functools
 import grp
 import json
 import logging
@@ -35,9 +36,12 @@ except ImportError:
     rpmutils_present = False
 import re
 import shutil
+import six
+import six.moves.configparser as ConfigParser
 import subprocess
 import tempfile
 
+
 # Override BOTO_CONFIG, which makes boto look only at the specified
 # config file, instead of the default locations
 os.environ['BOTO_CONFIG'] = '/var/lib/heat-cfntools/cfn-boto-cfg'
@@ -48,7 +52,7 @@ LOG = logging.getLogger(__name__)
 
 
 def to_boolean(b):
-    val = b.lower().strip() if isinstance(b, basestring) else b
+    val = b.lower().strip() if isinstance(b, six.string_types) else b
     return val in [True, 'true', 'yes', '1', 1]
 
 
@@ -136,7 +140,7 @@ class Hook(object):
     def event(self, ev_name, ev_object, ev_resource):
         if self.resource_name_get() == ev_resource and \
                 ev_name in self.triggers:
-            CommandRunner(self.action).run(user=self.runas)
+            CommandRunner(self.action, shell=True).run(user=self.runas)
         else:
             LOG.debug('event: {%s, %s, %s} did not match %s' %
                       (ev_name, ev_object, ev_resource, self.__str__()))
@@ -150,11 +154,39 @@ class Hook(object):
              self.action)
 
 
+class ControlledPrivilegesFailureException(Exception):
+    pass
+
+
+@contextlib.contextmanager
+def controlled_privileges(user):
+    orig_euid = None
+    try:
+        real = pwd.getpwnam(user)
+        if os.geteuid() != real.pw_uid:
+            orig_euid = os.geteuid()
+            os.seteuid(real.pw_uid)
+            LOG.debug("Privileges set for user %s" % user)
+    except Exception as e:
+        raise ControlledPrivilegesFailureException(e)
+
+    try:
+        yield
+    finally:
+        if orig_euid is not None:
+            try:
+                os.seteuid(orig_euid)
+                LOG.debug("Original privileges restored.")
+            except Exception as e:
+                LOG.error("Error restoring privileges %s" % e)
+
+
 class CommandRunner(object):
     """Helper class to run a command and store the output."""
 
-    def __init__(self, command, nextcommand=None):
+    def __init__(self, command, shell=False, nextcommand=None):
         self._command = command
+        self._shell = shell
         self._next = nextcommand
         self._stdout = None
         self._stderr = None
@@ -178,19 +210,34 @@ class CommandRunner(object):
             self
         """
         LOG.debug("Running command: %s" % self._command)
-        cmd = ['su', user, '-c', self._command]
-        subproc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
-                                   stderr=subprocess.PIPE, cwd=cwd, env=env)
-        output = subproc.communicate()
 
-        self._status = subproc.returncode
-        self._stdout = output[0]
-        self._stderr = output[1]
+        cmd = self._command
+        shell = self._shell
+
+        # Ensure commands that are given as string are run on shell
+        assert isinstance(cmd, six.string_types) is bool(shell)
+
+        try:
+            with controlled_privileges(user):
+                subproc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
+                                           stderr=subprocess.PIPE, cwd=cwd,
+                                           env=env, shell=shell)
+                output = subproc.communicate()
+                self._status = subproc.returncode
+                self._stdout = output[0]
+                self._stderr = output[1]
+        except ControlledPrivilegesFailureException as e:
+            LOG.error("Error setting privileges for user '%s': %s"
+                      % (user, e))
+            self._status = 126
+            self._stderr = six.text_type(e)
+
         if self._status:
             LOG.debug("Return code of %d after executing: '%s'\n"
                       "stdout: '%s'\n"
                       "stderr: '%s'" % (self._status, cmd, self._stdout,
                                         self._stderr))
+
         if self._next:
             self._next.run()
         return self
@@ -244,7 +291,7 @@ class RpmHelper(object):
                         e.g., ['2.0', '2.2', '2.2-1.fc16', '2.2.22-1.fc16']
         """
         if versions:
-            if isinstance(versions, basestring):
+            if isinstance(versions, six.string_types):
                 return versions
             versions = sorted(versions, rpmutils.compareVerOnly,
                               reverse=True)
@@ -273,7 +320,8 @@ class RpmHelper(object):
                    e.g., httpd-2.2.22
                    e.g., httpd-2.2.22-1.fc16
         """
-        command = CommandRunner("rpm -q %s" % pkg).run()
+        cmd = ['rpm', '-q', pkg]
+        command = CommandRunner(cmd).run()
         return command.status == 0
 
     @classmethod
@@ -286,8 +334,22 @@ class RpmHelper(object):
                    e.g., httpd-2.2.22
                    e.g., httpd-2.2.22-1.fc16
         """
-        cmd_str = "yum -y --showduplicates list available %s" % pkg
-        command = CommandRunner(cmd_str).run()
+        cmd = ['yum', '-y', '--showduplicates', 'list', 'available', pkg]
+        command = CommandRunner(cmd).run()
+        return command.status == 0
+
+    @classmethod
+    def dnf_package_available(cls, pkg):
+        """Indicates whether pkg is available via dnf.
+
+        Arguments:
+            pkg -- A package name (with optional version and release spec).
+                   e.g., httpd
+                   e.g., httpd-2.2.22
+                   e.g., httpd-2.2.22-1.fc21
+        """
+        cmd = ['dnf', '-y', '--showduplicates', 'list', 'available', pkg]
+        command = CommandRunner(cmd).run()
         return command.status == 0
 
     @classmethod
@@ -300,13 +362,13 @@ class RpmHelper(object):
                    e.g., httpd-2.2.22
                    e.g., httpd-2.2.22-1.fc16
         """
-        cmd_str = "zypper -n --no-refresh search %s" % pkg
-        command = CommandRunner(cmd_str).run()
+        cmd = ['zypper', '-n', '--no-refresh', 'search', pkg]
+        command = CommandRunner(cmd).run()
         return command.status == 0
 
     @classmethod
-    def install(cls, packages, rpms=True, zypper=False):
-        """Installs (or upgrades) a set of packages via RPM or via Yum.
+    def install(cls, packages, rpms=True, zypper=False, dnf=False):
+        """Installs (or upgrades) packages via RPM, yum, dnf, or zypper.
 
         Arguments:
             packages -- a list of packages to install
@@ -320,26 +382,30 @@ class RpmHelper(object):
                           - pkg name with version spec (httpd-2.2.22), or
                           - pkg name with version-release spec
                             (httpd-2.2.22-1.fc16)
+            zypper   -- if True:
+                        * overrides use of yum, use zypper instead
+            dnf      -- if True:
+                        * overrides use of yum, use dnf instead
+                        * packages must be in same format as yum pkg list
         """
         if rpms:
-            cmd = "rpm -U --force --nosignature "
-            cmd += " ".join(packages)
-            LOG.info("Installing packages: %s" % cmd)
+            cmd = ['rpm', '-U', '--force', '--nosignature']
         elif zypper:
-            cmd = "zypper -n install "
-            cmd += " ".join(packages)
-            LOG.info("Installing packages: %s" % cmd)
+            cmd = ['zypper', '-n', 'install']
+        elif dnf:
+            # use dnf --best to upgrade outdated-but-installed packages
+            cmd = ['dnf', '-y', '--best', 'install']
         else:
-            cmd = "yum -y install "
-            cmd += " ".join(packages)
-            LOG.info("Installing packages: %s" % cmd)
+            cmd = ['yum', '-y', 'install']
+        cmd.extend(packages)
+        LOG.info("Installing packages: %s" % cmd)
         command = CommandRunner(cmd).run()
         if command.status:
             LOG.warn("Failed to install packages: %s" % cmd)
 
     @classmethod
-    def downgrade(cls, packages, rpms=True, zypper=False):
-        """Downgrades a set of packages via RPM or via Yum.
+    def downgrade(cls, packages, rpms=True, zypper=False, dnf=False):
+        """Downgrades a set of packages via RPM, yum, dnf, or zypper.
 
         Arguments:
             packages -- a list of packages to downgrade
@@ -352,19 +418,28 @@ class RpmHelper(object):
                           - pkg name with version spec (httpd-2.2.22), or
                           - pkg name with version-release spec
                             (httpd-2.2.22-1.fc16)
+            dnf     -- if True:
+                       * Use dnf instead of RPM/yum
         """
         if rpms:
             cls.install(packages)
         elif zypper:
-            cmd = "zypper -n install --oldpackage "
-            cmd += " ".join(packages)
+            cmd = ['zypper', '-n', 'install', '--oldpackage']
+            cmd.extend(packages)
+            LOG.info("Downgrading packages: %s", cmd)
+            command = CommandRunner(cmd).run()
+            if command.status:
+                LOG.warn("Failed to downgrade packages: %s" % cmd)
+        elif dnf:
+            cmd = ['dnf', '-y', 'downgrade']
+            cmd.extend(packages)
             LOG.info("Downgrading packages: %s", cmd)
             command = CommandRunner(cmd).run()
             if command.status:
                 LOG.warn("Failed to downgrade packages: %s" % cmd)
         else:
-            cmd = "yum -y downgrade "
-            cmd += " ".join(packages)
+            cmd = ['yum', '-y', 'downgrade']
+            cmd.extend(packages)
             LOG.info("Downgrading packages: %s" % cmd)
             command = CommandRunner(cmd).run()
             if command.status:
@@ -374,7 +449,7 @@ class RpmHelper(object):
 class PackagesHandler(object):
     _packages = {}
 
-    _package_order = ["dpkg", "rpm", "apt", "yum"]
+    _package_order = ["dpkg", "rpm", "apt", "yum", "dnf"]
 
     @staticmethod
     def _pkgsort(pkg1, pkg2):
@@ -398,22 +473,23 @@ class PackagesHandler(object):
         # TODO(asalkeld) support versions
         # -b == local & remote install
         # -y == install deps
-        opts = '-b -y'
-        for pkg_name, versions in packages.iteritems():
+        opts = ['-b', '-y']
+        for pkg_name, versions in packages.items():
             if len(versions) > 0:
-                cmd_str = 'gem install %s --version %s %s' % (opts,
-                                                              versions[0],
-                                                              pkg_name)
-                CommandRunner(cmd_str).run()
+                cmd = ['gem', 'install'] + opts
+                cmd.extend(['--version', versions[0], pkg_name])
+                CommandRunner(cmd).run()
             else:
-                CommandRunner('gem install %s %s' % (opts, pkg_name)).run()
+                cmd = ['gem', 'install'] + opts
+                cmd.append(pkg_name)
+                CommandRunner(cmd).run()
 
     def _handle_python_packages(self, packages):
         """very basic support for easy_install."""
         # TODO(asalkeld) support versions
-        for pkg_name, versions in packages.iteritems():
-            cmd_str = 'easy_install %s' % (pkg_name)
-            CommandRunner(cmd_str).run()
+        for pkg_name, versions in packages.items():
+            cmd = ['easy_install', pkg_name]
+            CommandRunner(cmd).run()
 
     def _handle_zypper_packages(self, packages):
         """Handle installation, upgrade, or downgrade of packages via yum.
@@ -438,7 +514,7 @@ class PackagesHandler(object):
         # collect pkgs for batch processing at end
         installs = []
         downgrades = []
-        for pkg_name, versions in packages.iteritems():
+        for pkg_name, versions in packages.items():
             ver = RpmHelper.newest_rpm_version(versions)
             pkg = "%s-%s" % (pkg_name, ver) if ver else pkg_name
             if RpmHelper.rpm_package_installed(pkg):
@@ -460,8 +536,8 @@ class PackagesHandler(object):
         if downgrades:
             RpmHelper.downgrade(downgrades, zypper=True)
 
-    def _handle_yum_packages(self, packages):
-        """Handle installation, upgrade, or downgrade of packages via yum.
+    def _handle_dnf_packages(self, packages):
+        """Handle installation, upgrade, or downgrade of packages via dnf.
 
         Arguments:
         packages -- a package entries map of the form:
@@ -483,7 +559,63 @@ class PackagesHandler(object):
         # collect pkgs for batch processing at end
         installs = []
         downgrades = []
-        for pkg_name, versions in packages.iteritems():
+        for pkg_name, versions in packages.items():
+            ver = RpmHelper.newest_rpm_version(versions)
+            pkg = "%s-%s" % (pkg_name, ver) if ver else pkg_name
+            if RpmHelper.rpm_package_installed(pkg):
+                # FIXME:print non-error, but skipping pkg
+                pass
+            elif not RpmHelper.dnf_package_available(pkg):
+                LOG.warn("Skipping package '%s'. Not available via yum" % pkg)
+            elif not ver:
+                installs.append(pkg)
+            else:
+                current_ver = RpmHelper.rpm_package_version(pkg)
+                rc = RpmHelper.compare_rpm_versions(current_ver, ver)
+                if rc < 0:
+                    installs.append(pkg)
+                elif rc > 0:
+                    downgrades.append(pkg)
+        if installs:
+            RpmHelper.install(installs, rpms=False, dnf=True)
+        if downgrades:
+            RpmHelper.downgrade(downgrades, rpms=False, dnf=True)
+
+    def _handle_yum_packages(self, packages):
+        """Handle installation, upgrade, or downgrade of packages via yum.
+
+        Arguments:
+        packages -- a package entries map of the form:
+                      "pkg_name" : "version",
+                      "pkg_name" : ["v1", "v2"],
+                      "pkg_name" : []
+
+        For each package entry:
+          * if no version is supplied and the package is already installed, do
+            nothing
+          * if no version is supplied and the package is _not_ already
+            installed, install it
+          * if a version string is supplied, and the package is already
+            installed, determine whether to downgrade or upgrade (or do nothing
+            if version matches installed package)
+          * if a version array is supplied, choose the highest version from the
+            array and follow same logic for version string above
+        """
+
+        cmd = CommandRunner(['which', 'yum']).run()
+        if cmd.status == 1:
+            # yum not available, use DNF if available
+            self._handle_dnf_packages(packages)
+            return
+        elif cmd.status == 127:
+            # `which` command not found
+            LOG.info("`which` not found. Using yum without checking if dnf "
+                     "is available")
+
+        # collect pkgs for batch processing at end
+        installs = []
+        downgrades = []
+        for pkg_name, versions in packages.items():
             ver = RpmHelper.newest_rpm_version(versions)
             pkg = "%s-%s" % (pkg_name, ver) if ver else pkg_name
             if RpmHelper.rpm_package_installed(pkg):
@@ -523,14 +655,15 @@ class PackagesHandler(object):
     def _handle_apt_packages(self, packages):
         """very basic support for apt."""
         # TODO(asalkeld) support versions
-        pkg_list = ' '.join([p for p in packages])
+        pkg_list = list(packages)
 
-        cmd_str = 'DEBIAN_FRONTEND=noninteractive apt-get -y install %s' % \
-                  pkg_list
-        CommandRunner(cmd_str).run()
+        env = {'DEBIAN_FRONTEND': 'noninteractive'}
+        cmd = ['apt-get', '-y', 'install'] + pkg_list
+        CommandRunner(cmd).run(env=env)
 
     # map of function pointers to handle different package managers
     _package_handlers = {"yum": _handle_yum_packages,
+                         "dnf": _handle_dnf_packages,
                          "zypper": _handle_zypper_packages,
                          "rpm": _handle_rpm_packages,
                          "apt": _handle_apt_packages,
@@ -552,10 +685,19 @@ class PackagesHandler(object):
           * rpm
           * apt
           * yum
+          * dnf
         """
         if not self._packages:
             return
-        packages = sorted(self._packages.iteritems(), PackagesHandler._pkgsort)
+        try:
+            packages = sorted(
+                self._packages.items(), cmp=PackagesHandler._pkgsort)
+        except TypeError:
+            # On Python 3, we have to use key instead of cmp
+            # This could also work on Python 2.7, but not on 2.6
+            packages = sorted(
+                self._packages.items(),
+                key=functools.cmp_to_key(PackagesHandler._pkgsort))
 
         for manager, package_entries in packages:
             handler = self._package_handler(manager)
@@ -572,7 +714,7 @@ class FilesHandler(object):
     def apply_files(self):
         if not self._files:
             return
-        for fdest, meta in self._files.iteritems():
+        for fdest, meta in self._files.items():
             dest = fdest.encode()
             try:
                 os.makedirs(os.path.dirname(dest))
@@ -583,16 +725,17 @@ class FilesHandler(object):
                     LOG.exception(e)
 
             if 'content' in meta:
-                if isinstance(meta['content'], basestring):
+                if isinstance(meta['content'], six.string_types):
                     f = open(dest, 'w+')
                     f.write(meta['content'])
                     f.close()
                 else:
                     f = open(dest, 'w+')
-                    f.write(json.dumps(meta['content'], indent=4))
+                    f.write(json.dumps(meta['content'], indent=4)
+                     .encode('UTF-8'))
                     f.close()
             elif 'source' in meta:
-                CommandRunner('curl -o %s %s' % (dest, meta['source'])).run()
+                CommandRunner(['curl', '-o', dest, meta['source']]).run()
             else:
                 LOG.error('%s %s' % (dest, str(meta)))
                 continue
@@ -693,14 +836,15 @@ class SourcesHandler(object):
 
     def _apply_source(self, dest, url):
         cmd = self._apply_source_cmd(dest, url)
+        #FIXME bug 1498298
         if cmd != '':
-            runner = CommandRunner(cmd)
+            runner = CommandRunner(cmd, shell=True)
             runner.run()
 
     def apply_sources(self):
         if not self._sources:
             return
-        for dest, url in self._sources.iteritems():
+        for dest, url in self._sources.items():
             self._apply_source(dest, url)
 
 
@@ -716,47 +860,52 @@ class ServicesHandler(object):
         if os.path.exists("/bin/systemctl"):
             service_exe = "/bin/systemctl"
             service = '%s.service' % service
-            service_start = '%s start %s'
-            service_status = '%s status %s'
-            service_stop = '%s stop %s'
+            service_start = [service_exe, 'start', service]
+            service_status = [service_exe, 'status', service]
+            service_stop = [service_exe, 'stop', service]
         elif os.path.exists("/sbin/service"):
             service_exe = "/sbin/service"
-            service_start = '%s %s start'
-            service_status = '%s %s status'
-            service_stop = '%s %s stop'
+            service_start = [service_exe, service, 'start']
+            service_status = [service_exe, service, 'status']
+            service_stop = [service_exe, service, 'stop']
         else:
             service_exe = "/usr/sbin/service"
-            service_start = '%s %s start'
-            service_status = '%s %s status'
-            service_stop = '%s %s stop'
+            service_start = [service_exe, service, 'start']
+            service_status = [service_exe, service, 'status']
+            service_stop = [service_exe, service, 'stop']
 
         if os.path.exists("/bin/systemctl"):
             enable_exe = "/bin/systemctl"
-            enable_on = '%s enable %s'
-            enable_off = '%s disable %s'
+            enable_on = [enable_exe, 'enable', service]
+            enable_off = [enable_exe, 'disable', service]
         elif os.path.exists("/sbin/chkconfig"):
             enable_exe = "/sbin/chkconfig"
-            enable_on = '%s %s on'
-            enable_off = '%s %s off'
+            enable_on = [enable_exe, service, 'on']
+            enable_off = [enable_exe, service, 'off']
+
         else:
             enable_exe = "/usr/sbin/update-rc.d"
-            enable_on = '%s %s enable'
-            enable_off = '%s %s disable'
+            enable_on = [enable_exe, service, 'enable']
+            enable_off = [enable_exe, service, 'disable']
 
-        cmd = ""
+        cmd = None
         if "enable" == command:
-            cmd = enable_on % (enable_exe, service)
+            cmd = enable_on
         elif "disable" == command:
-            cmd = enable_off % (enable_exe, service)
+            cmd = enable_off
         elif "start" == command:
-            cmd = service_start % (service_exe, service)
+            cmd = service_start
         elif "stop" == command:
-            cmd = service_stop % (service_exe, service)
+            cmd = service_stop
         elif "status" == command:
-            cmd = service_status % (service_exe, service)
-        command = CommandRunner(cmd)
-        command.run()
-        return command
+            cmd = service_status
+
+        if cmd is not None:
+            command = CommandRunner(cmd)
+            command.run()
+            return command
+        else:
+            LOG.error("Unknown sysv command %s" % command)
 
     def _initialize_service(self, handler, service, properties):
         if "enabled" in properties:
@@ -790,16 +939,15 @@ class ServicesHandler(object):
                 if start_cmd.status != 0:
                     LOG.warning('Service %s did not start. STDERR: %s' %
                                (service, start_cmd.stderr))
-                    return
                 for h in self.hooks:
                     h.event('service.restarted', service, self.resource)
 
     def _monitor_services(self, handler, services):
-        for service, properties in services.iteritems():
+        for service, properties in services.items():
             self._monitor_service(handler, service, properties)
 
     def _initialize_services(self, handler, services):
-        for service, properties in services.iteritems():
+        for service, properties in services.items():
             self._initialize_service(handler, service, properties)
 
     # map of function pointers to various service handlers
@@ -818,7 +966,7 @@ class ServicesHandler(object):
         """Starts, stops, enables, disables services."""
         if not self._services:
             return
-        for manager, service_entries in self._services.iteritems():
+        for manager, service_entries in self._services.items():
             handler = self._service_handler(manager)
             if not handler:
                 LOG.warn("Skipping invalid service type: %s" % manager)
@@ -829,7 +977,7 @@ class ServicesHandler(object):
         """Restarts failed services, and runs hooks."""
         if not self._services:
             return
-        for manager, service_entries in self._services.iteritems():
+        for manager, service_entries in self._services.items():
             handler = self._service_handler(manager)
             if not handler:
                 LOG.warn("Skipping invalid service type: %s" % manager)
@@ -940,7 +1088,7 @@ class CommandsHandler(object):
                 return
 
         if "test" in properties:
-            test = CommandRunner(properties["test"])
+            test = CommandRunner(properties["test"], shell=True)
             test_status = test.run('root', cwd, env).status
             if test_status != 0:
                 LOG.info("%s test returns false, skipping command"
@@ -952,10 +1100,8 @@ class CommandsHandler(object):
         if "command" in properties:
             try:
                 command = properties["command"]
-                if isinstance(command, list):
-                    escape = lambda x: '"%s"' % x.replace('"', '\\"')
-                    command = ' '.join(map(escape, command))
-                command = CommandRunner(command)
+                shell = isinstance(command, six.string_types)
+                command = CommandRunner(command, shell=shell)
                 command.run('root', cwd, env)
                 command_status = command.status
             except OSError as e:
@@ -988,20 +1134,17 @@ class GroupsHandler(object):
         """Create Linux/UNIX groups and assign group IDs."""
         if not self.groups:
             return
-        for group, properties in self.groups.iteritems():
+        for group, properties in self.groups.items():
             LOG.debug("%s group is being created" % group)
             self._initialize_group(group, properties)
 
     def _initialize_group(self, group, properties):
         gid = properties.get("gid", None)
-
-        param_list = []
-        param_list.append(group)
-
+        cmd = ['groupadd', group]
         if gid is not None:
-            param_list.append("--gid " + gid)
+            cmd.extend(['--gid', str(gid)])
 
-        command = CommandRunner("groupadd " + ' '.join(param_list))
+        command = CommandRunner(cmd)
         command.run()
         command_status = command.status
 
@@ -1032,7 +1175,7 @@ class UsersHandler(object):
         """Create Linux/UNIX users and assign user IDs, groups and homedir."""
         if not self.users:
             return
-        for user, properties in self.users.iteritems():
+        for user, properties in self.users.items():
             LOG.debug("%s user is being created" % user)
             self._initialize_user(user, properties)
 
@@ -1040,23 +1183,23 @@ class UsersHandler(object):
         uid = properties.get("uid", None)
         homeDir = properties.get("homeDir", None)
 
-        param_list = []
-        param_list.append(user)
+        cmd = ['useradd', user]
 
         if uid is not None:
-            param_list.append("--uid " + uid)
+            cmd.extend(['--uid', six.text_type(uid)])
 
         if homeDir is not None:
-            param_list.append("--home " + homeDir)
+            cmd.extend(['--home', six.text_type(homeDir)])
 
         if "groups" in properties:
-            param_list.append("--groups " + ','.join(properties["groups"]))
+            groups = ','.join(properties["groups"])
+            cmd.extend(['--groups', groups])
 
         #Users are created as non-interactive system users with a shell
         #of /sbin/nologin. This is by design and cannot be modified.
-        param_list.append("--shell /sbin/nologin")
+        cmd.extend(['--shell', '/sbin/nologin'])
 
-        command = CommandRunner("useradd " + ' '.join(param_list))
+        command = CommandRunner(cmd)
         command.run()
         command_status = command.status
 
@@ -1149,7 +1292,8 @@ class Metadata(object):
 
         url = 'http://169.254.169.254/openstack/2012-08-10/meta_data.json'
         if not os.path.exists(cache_path):
-            CommandRunner('curl -o %s %s' % (cache_path, url)).run()
+            cmd = ['curl', '-o', cache_path, url]
+            CommandRunner(cmd).run()
         try:
             with open(cache_path) as fd:
                 try:
@@ -1267,7 +1411,7 @@ class Metadata(object):
                                          mode='wb',
                                          delete=False) as cf:
             os.chmod(cf.name, 0o600)
-            cf.write(json.dumps(self._metadata))
+            cf.write(json.dumps(self._metadata).encode('UTF-8'))
             os.rename(cf.name, last_path)
             cf.close()
             if res_last_path != last_path:
